import inspect
import typing

from pydantic.fields import ModelField
from starlette.datastructures import FormData, UploadFile
from starlette.requests import Request

from xpresso._utils.compat import get_args
from xpresso._utils.typing import Some, model_field_from_param
from xpresso.binders._body.extractors.body_field_validation import validate_body_field
from xpresso.binders._body.extractors.form_encoded_field import (
    FormEncodedFieldExtractorMarker,
)
from xpresso.binders._body.form_field import FormDataExtractor, FormFieldMarker
from xpresso.binders._body.media_type_validator import MediaTypeValidator
from xpresso.binders._body.media_type_validator import (
    get_validator as get_media_type_validator,
)
from xpresso.binders.api import BodyExtractor


class FormDataBodyExtractor:
    __slots__ = ("media_type_validator", "field", "field_extractors")

    def __init__(
        self,
        media_type_validator: MediaTypeValidator,
        field: ModelField,
        field_extractors: typing.Mapping[str, FormDataExtractor],
    ) -> None:
        self.media_type_validator = media_type_validator
        self.field = field
        self.field_extractors = field_extractors

    def matches_media_type(self, media_type: typing.Optional[str]) -> bool:
        return self.media_type_validator.matches(media_type)

    async def extract_from_request(self, request: Request) -> typing.Any:
        media_type = request.headers.get("content-type", None)
        self.media_type_validator.validate(media_type, loc=("body",))
        if (
            request.headers.get("Content-Length", None) == "0"
            and self.field.required is not True
        ):
            # this is the only way to know the body is empty
            return validate_body_field(
                None,
                field=self.field,
                loc=("body",),
            )
        return validate_body_field(
            Some(await self._extract(await request.form(), loc=("body",))),
            field=self.field,
            loc=("body",),
        )

    async def _extract(
        self, form: FormData, loc: typing.Iterable[typing.Union[str, int]]
    ) -> typing.Any:
        res: typing.Dict[str, typing.Any] = {}
        for param_name, extractor in self.field_extractors.items():
            extracted = await extractor.extract_from_form(form, loc=loc)
            if isinstance(extracted, Some):
                res[param_name] = extracted.value
        return res

    async def extract_from_field(
        self,
        field: typing.Union[str, UploadFile],
        *,
        loc: typing.Iterable[typing.Union[str, int]],
    ) -> typing.Any:
        raise NotImplementedError


class FormDataBodyExtractorMarker(typing.NamedTuple):
    enforce_media_type: bool
    media_type: str

    def register_parameter(self, param: inspect.Parameter) -> BodyExtractor:
        form_data_field = model_field_from_param(param)

        field_extractors: typing.Dict[str, FormDataExtractor] = {}
        # use pydantic to get rid of outer annotated, optional, etc.
        # use pydantic to get rid of outer annotated, optional, etc.
        model = form_data_field.type_
        for field_param in inspect.signature(model).parameters.values():
            for m in get_args(field_param.annotation):
                if isinstance(m, FormFieldMarker):
                    field_extractor = m.extractor_marker.register_parameter(field_param)
                    break
            else:
                field_extractor = FormEncodedFieldExtractorMarker(
                    alias=None,
                    style="form",
                    explode=True,
                ).register_parameter(field_param)
            field_extractors[field_param.name] = field_extractor
        if self.enforce_media_type and self.media_type:
            media_type_validator = get_media_type_validator(self.media_type)
        else:
            media_type_validator = get_media_type_validator(None)
        return FormDataBodyExtractor(
            media_type_validator=media_type_validator,
            field_extractors=field_extractors,
            field=form_data_field,
        )



import inspect
import typing
from enum import Enum

from pydantic import BaseConfig, BaseModel
from pydantic.fields import (
    MAPPING_LIKE_SHAPES,
    SHAPE_FROZENSET,
    SHAPE_LIST,
    SHAPE_SEQUENCE,
    SHAPE_SET,
    SHAPE_TUPLE,
    SHAPE_TUPLE_ELLIPSIS,
    ModelField,
)
from pydantic.schema import TypeModelOrEnum
from pydantic.schema import get_model_name_map as get_model_name_map_pydantic

T = typing.TypeVar("T")


def model_field_from_param(param: inspect.Parameter) -> ModelField:
    return ModelField.infer(
        name=param.name,
        value=param.default if param.default is not param.empty else ...,
        annotation=param.annotation,
        class_validators={},
        config=BaseConfig,
    )


def filter_pydantic_models_from_set(
    s: typing.AbstractSet[typing.Any],
) -> typing.Set[TypeModelOrEnum]:
    def f(x: typing.Any) -> bool:
        return inspect.isclass(x) and issubclass(x, (BaseModel, Enum))

    return set(filter(f, s))


def filter_pydantic_models_from_mapping(
    m: typing.Mapping[typing.Any, T]
) -> typing.Dict[TypeModelOrEnum, T]:
    keys = filter_pydantic_models_from_set(m.keys())
    return {k: m[k] for k in keys}


def get_model_name_map(unique_models: typing.Set[type]) -> typing.Dict[type, str]:
    # this works with any class, but Pydantic types it as if it isn't
    # if this at some point breaks, we'll just implement it in this function
    return get_model_name_map_pydantic(unique_models)  # type: ignore[arg-type]


def is_sequence_like(field: ModelField) -> bool:
    return field.shape in (
        SHAPE_TUPLE,
        SHAPE_TUPLE_ELLIPSIS,
        SHAPE_LIST,
        SHAPE_SET,
        SHAPE_FROZENSET,
        SHAPE_LIST,
        SHAPE_SEQUENCE,
    )


def is_mapping_like(field: ModelField) -> bool:
    return (
        field.shape in MAPPING_LIKE_SHAPES
        or inspect.isclass(field.type_)
        and issubclass(field.type_, BaseModel)
    )


class Some(typing.Generic[T]):
    __slots__ = ("value",)

    def __init__(self, value: T) -> None:
        self.value = value

    def __repr__(self) -> str:
        return f"Some({repr(self.value)})"



import typing


async def convert_stream_to_bytes(
    stream: typing.AsyncGenerator[bytes, None]
) -> typing.Optional[bytes]:
    data: typing.List[bytes] = []
    async for chunk in stream:  # pragma: no cover # stream always has at least 1 chunk
        data.append(chunk)
    if len(data) == 1 and data[0] == b"":
        # no content received
        return None
    return b"".join(data)



import inspect
import typing

from pydantic.error_wrappers import ErrorWrapper
from pydantic.fields import ModelField
from starlette.datastructures import UploadFile
from starlette.requests import Request

from xpresso._utils.compat import Protocol
from xpresso._utils.typing import Some, model_field_from_param
from xpresso.binders._body.extractors.body_field_validation import validate_body_field
from xpresso.binders._body.media_type_validator import MediaTypeValidator
from xpresso.binders._body.media_type_validator import (
    get_validator as get_media_type_validator,
)
from xpresso.binders._utils.stream_to_bytes import convert_stream_to_bytes
from xpresso.binders.api import BodyExtractor
from xpresso.exceptions import RequestValidationError


class Decoder(Protocol):
    def __call__(self, s: typing.Union[str, bytes]) -> typing.Any:
        ...


class JsonBodyExtractor:
    __slots__ = ("field", "decoder", "media_type_validator", "consume")

    def __init__(
        self,
        field: ModelField,
        decoder: Decoder,
        media_type_validator: MediaTypeValidator,
        consume: bool,
    ) -> None:
        self.field = field
        self.decoder = decoder
        self.media_type_validator = media_type_validator
        self.consume = consume

    def matches_media_type(self, media_type: typing.Optional[str]) -> bool:
        return self.media_type_validator.matches(media_type)

    async def extract_from_request(self, request: Request) -> typing.Any:
        loc = ("body",)
        self.media_type_validator.validate(
            request.headers.get("content-type", None), loc=loc
        )
        if self.consume:
            data_from_stream = await convert_stream_to_bytes(request.stream())
            if data_from_stream is None:
                return validate_body_field(None, field=self.field, loc=loc)
        else:
            data_from_stream = await request.body()
        return validate_body_field(
            Some(self._decode(data_from_stream, loc=loc)),
            field=self.field,
            loc=loc,
        )

    async def extract_from_field(
        self,
        field: typing.Union[str, UploadFile],
        *,
        loc: typing.Iterable[typing.Union[str, int]],
    ) -> typing.Any:
        if isinstance(field, UploadFile):
            return self._decode(await field.read(), loc=loc)
        return self._decode(field, loc=loc)

    def _decode(
        self,
        value: typing.Union[str, bytes],
        loc: typing.Iterable[typing.Union[int, str]],
    ) -> typing.Union[bytes, UploadFile]:
        try:
            decoded = self.decoder(value)
        except Exception as e:
            raise RequestValidationError(
                [
                    ErrorWrapper(
                        exc=TypeError("Data is not valid JSON"),
                        loc=tuple(loc),
                    )
                ]
            ) from e
        return decoded


class JsonBodyExtractorMarker(typing.NamedTuple):
    decoder: Decoder
    enforce_media_type: bool
    consume: bool

    def register_parameter(self, param: inspect.Parameter) -> BodyExtractor:
        if self.enforce_media_type:
            media_type_validator = get_media_type_validator("application/json")
        else:
            media_type_validator = get_media_type_validator(None)
        return JsonBodyExtractor(
            field=model_field_from_param(param),
            decoder=self.decoder,
            media_type_validator=media_type_validator,
            consume=self.consume,
        )



from __future__ import annotations

import typing

import di
from di.api.providers import DependencyProvider
from di.dependant import Injectable as InjectableBase

from xpresso._utils.compat import Literal

T = typing.TypeVar("T")


Scope = Literal["app", "connection", "endpoint"]
Scopes: typing.Tuple[Literal["app"], Literal["connection"], Literal["endpoint"]] = (
    "app",
    "connection",
    "endpoint",
)


class Depends(di.Marker, di.Dependant[typing.Any]):
    scope: Scope

    def __init__(
        self,
        call: typing.Optional[DependencyProvider] = None,
        scope: Scope = "connection",
        use_cache: bool = True,
        wire: bool = True,
        sync_to_thread: bool = False,
    ) -> None:
        super().__init__(
            call=call,
            scope=scope,
            use_cache=use_cache,
            wire=wire,
            sync_to_thread=sync_to_thread,
        )


class Injectable(InjectableBase):
    __slots__ = ()

    def __init_subclass__(
        cls,
        call: typing.Optional[DependencyProvider] = None,
        scope: Scope = "connection",
        use_cache: bool = True,
        **kwargs: typing.Any,
    ) -> None:
        return super().__init_subclass__(call, scope, use_cache, **kwargs)


class Singleton(InjectableBase):
    __slots__ = ()

    def __init_subclass__(
        cls,
        call: typing.Optional[DependencyProvider] = None,
        scope: Scope = "app",
        use_cache: bool = True,
        **kwargs: typing.Any,
    ) -> None:
        return super().__init_subclass__(call, scope, use_cache, **kwargs)



import inspect
import typing

from xpresso._utils.compat import Literal, get_args
from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.form_field import FormFieldMarker, FormFieldOpenAPIProvider
from xpresso.binders._body.openapi.form_encoded_field import OpenAPIFormFieldMarker
from xpresso.binders._utils.examples import parse_examples
from xpresso.binders.api import ModelNameMap, OpenAPIBody, Schemas
from xpresso.openapi import models as openapi_models


class OpenAPIFormDataBody(typing.NamedTuple):
    field_openapi_providers: typing.Mapping[str, FormFieldOpenAPIProvider]
    required_fields: typing.List[str]
    description: typing.Optional[str]
    examples: typing.Optional[openapi_models.Examples]
    media_type: Literal[
        "multipart/form-data",
        "application/x-www-form-urlencoded",
    ]
    required: bool
    nullable: bool
    include_in_schema: bool

    def get_models(self) -> typing.List[type]:
        return [
            model
            for provider in self.field_openapi_providers.values()
            for model in provider.get_models()
        ]

    def get_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Schema:
        properties = {
            field_name: field_openapi.get_field_schema(
                model_name_map=model_name_map, schemas=schemas
            )
            for field_name, field_openapi in self.field_openapi_providers.items()
        }
        return openapi_models.Schema(
            type="object",
            properties=properties,
            required=self.required_fields or None,
            nullable=self.nullable or None,
        )

    def get_openapi_media_type(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.MediaType:
        encodings: typing.Dict[str, openapi_models.Encoding] = {}
        for field_name, field_openapi in self.field_openapi_providers.items():
            encoding = field_openapi.get_field_encoding(model_name_map, schemas)
            encodings[field_name] = encoding
        return openapi_models.MediaType(
            schema=self.get_schema(model_name_map=model_name_map, schemas=schemas),
            examples=self.examples,  # type: ignore[arg-type]
            encoding=encodings or None,
        )

    def get_openapi_body(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.RequestBody:
        return openapi_models.RequestBody(
            description=self.description,
            required=self.required,
            content={
                self.media_type: self.get_openapi_media_type(model_name_map, schemas)
            },
        )

    # Nested forms are not supported
    def get_field_encoding(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Encoding:
        raise NotImplementedError

    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Schema:
        raise NotImplementedError


class OpenAPIFormDataMarker(typing.NamedTuple):
    description: typing.Optional[str]
    examples: typing.Optional[
        typing.Dict[str, typing.Union[openapi_models.Example, typing.Any]]
    ]
    media_type: Literal[
        "multipart/form-data",
        "application/x-www-form-urlencoded",
    ]
    include_in_schema: bool

    def register_parameter(self, param: inspect.Parameter) -> OpenAPIBody:
        form_data_field = model_field_from_param(param)
        required = form_data_field.required is not False
        field_openapi_providers: typing.Dict[str, FormFieldOpenAPIProvider] = {}
        required_fields: typing.List[str] = []
        # use pydantic to get rid of outer annotated, optional, etc.
        model = form_data_field.type_
        for field_param in inspect.signature(model).parameters.values():
            for m in get_args(field_param.annotation):
                if isinstance(m, FormFieldMarker):
                    field_openapi = m.openapi_marker.register_parameter(field_param)
                    break
            else:
                field_openapi = OpenAPIFormFieldMarker(
                    alias=None,
                    style="form",
                    explode=True,
                    include_in_schema=True,
                ).register_parameter(field_param)
            field_name = field_openapi.field_name
            if field_openapi.include_in_schema:
                field_openapi_providers[field_name] = field_openapi
                field = model_field_from_param(field_param)
                if field.required is not False:
                    required_fields.append(field_name)
        examples = parse_examples(self.examples) if self.examples else None
        return OpenAPIFormDataBody(
            field_openapi_providers=field_openapi_providers,
            required_fields=required_fields,
            description=self.description,
            examples=examples,
            media_type=self.media_type,
            required=required,
            nullable=form_data_field.allow_none,
            include_in_schema=self.include_in_schema,
        )



import inspect
import typing

from di.typing import get_markers_from_annotation
from starlette.datastructures import UploadFile
from starlette.requests import Request

from xpresso._utils.compat import Annotated, get_args, get_origin
from xpresso.binders.api import BodyExtractor
from xpresso.binders.dependants import BodyBinderMarker
from xpresso.exceptions import HTTPException


class ContentTypeDiscriminatedExtractor:
    __slots__ = ("sub_body_extractors",)

    def __init__(
        self,
        sub_body_extractors: typing.Iterable[BodyExtractor],
    ) -> None:
        self.sub_body_extractors = sub_body_extractors

    async def extract_from_request(self, request: Request) -> typing.Any:
        media_type = request.headers.get("content-type", None)
        for sub_body_extractor in self.sub_body_extractors:
            if sub_body_extractor.matches_media_type(media_type):
                return await sub_body_extractor.extract_from_request(request)
        if media_type:
            raise HTTPException(
                status_code=415, detail=f"Media type {media_type} is not acceptable"
            )
        else:
            raise HTTPException(status_code=415, detail="Content-Type header missing")

    def matches_media_type(self, media_type: typing.Optional[str]) -> bool:
        raise NotImplementedError

    async def extract_from_field(
        self,
        field: typing.Union[str, UploadFile],
        *,
        loc: typing.Iterable[typing.Union[str, int]],
    ) -> typing.Any:
        raise NotImplementedError


class ContentTypeDiscriminatedExtractorMarker(typing.NamedTuple):
    def register_parameter(self, param: inspect.Parameter) -> BodyExtractor:
        sub_body_extractors: typing.List[BodyExtractor] = []

        annotation = param.annotation
        origin = get_origin(annotation)
        assert origin is Annotated
        annotation = next(iter(get_args(annotation)))
        origin = get_origin(annotation)
        if origin is not typing.Union:
            raise TypeError("Unioned bodies must be a Union of simple bodies")
        args = get_args(annotation)
        for arg in args:
            sub_body_param = inspect.Parameter(
                name=param.name,
                kind=param.kind,
                annotation=arg,
                default=param.default,
            )
            marker = next(
                get_markers_from_annotation(
                    sub_body_param.annotation, BodyBinderMarker
                ),
                None,
            )
            if marker is None:
                raise TypeError(f"Type annotation is missing body marker: {arg}")
            sub_body_extractor = marker.extractor_marker
            provider = sub_body_extractor.register_parameter(sub_body_param)
            sub_body_extractors.append(provider)
        return ContentTypeDiscriminatedExtractor(
            sub_body_extractors=sub_body_extractors
        )



import inspect
import typing

from di.typing import get_markers_from_annotation

from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.form_field import FormFieldOpenAPIProvider
from xpresso.binders.api import ModelNameMap, OpenAPIBody, Schemas
from xpresso.binders.dependants import BodyBinderMarker
from xpresso.openapi import models

T = typing.TypeVar("T")


class _Base(typing.NamedTuple):
    field_name: str
    include_in_schema: bool
    field_openapi: OpenAPIBody

    def get_models(self) -> typing.List[type]:
        return self.field_openapi.get_models()

    def get_field_encoding(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> models.Encoding:
        return self.field_openapi.get_field_encoding(model_name_map, schemas)


class OpenAPIField(_Base):
    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> models.Schema:
        return self.field_openapi.get_field_schema(
            model_name_map=model_name_map, schemas=schemas
        )


class OpenAPIRepeatedField(_Base):
    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> models.Schema:
        return models.Schema(
            type="array",
            items=self.field_openapi.get_field_schema(
                model_name_map=model_name_map, schemas=schemas
            ),
        )


class OpenAPIFieldMarker(typing.NamedTuple):
    alias: typing.Optional[str]
    include_in_schema: bool
    repeated: bool

    def register_parameter(self, param: inspect.Parameter) -> FormFieldOpenAPIProvider:
        marker = next(
            get_markers_from_annotation(param.annotation, BodyBinderMarker), None
        )
        if marker is None:
            raise TypeError(
                "No field marker found"
                "\n You must include a valid field marker using ExtractField[AsJson[...]]"
                " or Annotated[..., Json(), Field()]"
            )
        openapi_provider = marker.openapi_marker.register_parameter(param)
        field_name = self.alias or model_field_from_param(param).alias
        if self.repeated:
            return OpenAPIRepeatedField(
                field_name=field_name,
                field_openapi=openapi_provider,
                include_in_schema=self.include_in_schema,
            )
        return OpenAPIField(
            field_name=field_name,
            field_openapi=openapi_provider,
            include_in_schema=self.include_in_schema,
        )



import inspect
import typing

from pydantic.fields import ModelField
from pydantic.schema import get_flat_models_from_field

from xpresso._utils.schemas import openapi_schema_from_pydantic_field
from xpresso._utils.typing import model_field_from_param
from xpresso.binders._body.form_field import FormFieldOpenAPIProvider
from xpresso.binders.api import ModelNameMap, Schemas
from xpresso.openapi import models as openapi_models


class OpenAPIFormEncodedField(typing.NamedTuple):
    field_name: str
    include_in_schema: bool
    alias: typing.Optional[str]
    style: str
    explode: bool
    field: ModelField

    def get_models(self) -> typing.List[type]:
        return list(get_flat_models_from_field(self.field, set()))

    def get_field_schema(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Schema:
        return openapi_schema_from_pydantic_field(self.field, model_name_map, schemas)

    def get_field_encoding(
        self, model_name_map: ModelNameMap, schemas: Schemas
    ) -> openapi_models.Encoding:
        return openapi_models.Encoding(
            contentType=None,
            style=self.style,
            explode=self.explode,
        )


class OpenAPIFormFieldMarker(typing.NamedTuple):
    alias: typing.Optional[str]
    style: str
    explode: bool
    include_in_schema: bool

    def register_parameter(self, param: inspect.Parameter) -> FormFieldOpenAPIProvider:
        field = model_field_from_param(param)
        field_name = self.alias or field.alias
        return OpenAPIFormEncodedField(
            alias=self.alias,
            style=self.style,
            explode=self.explode,
            field_name=field_name,
            field=field,
            include_in_schema=self.include_in_schema,
        )



import typing

import starlette.middleware
from di.api.dependencies import DependantBase
from starlette.routing import BaseRoute
from starlette.routing import Router as StarletteRouter
from starlette.types import Receive, Scope, Send

from xpresso._utils.compat import Protocol
from xpresso.responses import Responses


class _ASGIApp(Protocol):
    def __call__(
        self,
        scope: Scope,
        receive: Receive,
        send: Send,
    ) -> typing.Awaitable[None]:
        ...


_MiddlewareIterator = typing.Iterable[
    typing.Tuple[typing.Callable[..., _ASGIApp], typing.Mapping[str, typing.Any]]
]


class Router:
    routes: typing.Sequence[BaseRoute]
    lifespan: typing.Optional[typing.Callable[..., typing.AsyncContextManager[None]]]
    dependencies: typing.Sequence[DependantBase[typing.Any]]
    tags: typing.Sequence[str]
    responses: Responses
    include_in_schema: bool
    _app: _ASGIApp

    __slots__ = (
        "_app",
        "_router",
        "dependencies",
        "include_in_schema",
        "lifespan",
        "responses",
        "routes",
        "tags",
    )

    def __init__(
        self,
        routes: typing.Sequence[BaseRoute],
        *,
        middleware: typing.Optional[
            typing.Sequence[starlette.middleware.Middleware]
        ] = None,
        lifespan: typing.Optional[
            typing.Callable[..., typing.AsyncContextManager[None]]
        ] = None,
        redirect_slashes: bool = True,
        default: typing.Optional[_ASGIApp] = None,
        dependencies: typing.Optional[
            typing.Sequence[DependantBase[typing.Any]]
        ] = None,
        tags: typing.Optional[typing.List[str]] = None,
        responses: typing.Optional[Responses] = None,
        include_in_schema: bool = True,
    ) -> None:
        self.routes = list(routes)
        self.lifespan = lifespan
        self._router = StarletteRouter(
            routes=self.routes,
            redirect_slashes=redirect_slashes,
            default=default,  # type: ignore[arg-type]
            lifespan=lifespan,  # type: ignore[arg-type]
        )
        self.dependencies = list(dependencies or [])
        self.tags = list(tags or [])
        self.responses = dict(responses or {})
        self.include_in_schema = include_in_schema
        self._app = self._router.__call__
        if middleware is not None:
            for cls, options in typing.cast(_MiddlewareIterator, reversed(middleware)):
                self._app = cls(app=self._app, **options)

    def __call__(
        self,
        scope: Scope,
        receive: Receive,
        send: Send,
    ) -> typing.Awaitable[None]:
        return self._app(scope, receive, send)



from pydantic.fields import ModelField
from pydantic.schema import field_schema

from xpresso._utils.typing import filter_pydantic_models_from_mapping
from xpresso.binders.api import ModelNameMap, Schemas
from xpresso.openapi import models as openapi_models
from xpresso.openapi.constants import REF_PREFIX


def openapi_schema_from_pydantic_field(
    field: ModelField,
    model_name_map: ModelNameMap,
    schemas: Schemas,
) -> openapi_models.Schema:
    schema, refs, _ = field_schema(
        field,
        by_alias=True,
        ref_prefix=REF_PREFIX,
        model_name_map=filter_pydantic_models_from_mapping(model_name_map),
    )
    schemas.update(refs)
    return openapi_models.Schema(**schema, nullable=field.allow_none or None)
